<div id="problem_{{ element_id }}"
     class="problems-wrapper"
     role="group"
     aria-labelledby="{{ element_id }}-problem-title"
     data-problem-id="{{ id }}"
     data-url="{{ ajax_url }}"
     data-problem-score="{{ current_score }}"
     data-problem-total-possible="{{ total_possible }}"
     data-attempts-used="{{ attempts_used }}"
     data-content="{{ content }}"
     data-graded="{{ graded }}">
    <p class="loading-spinner">
        <i class="fa fa-spinner fa-pulse fa-2x fa-fw"></i>
        <span class="sr">Loading&hellip;</span>
    </p>
</div>
