.protex-absolute-panel {
   background-color: #BBBBBB;
}

.protex-canvas {
	outline: none;
}

.protex-caption-panel {
	padding: 10px 10px 10px 10px;
	margin: 0px 0px 0px 0px;
	border:2px solid #4E4E4E;
    font: normal normal normal 14pt/normal 'Open Sans', sans-serif;
	background-color: #B2B2FF;
}

.protex-textbox {
    font: normal normal normal 12pt/normal 'Open Sans', sans-serif !important;
    padding: 2px 2px 2px 2px !important;
}

.protex-caption-panel-ss-bonds-on {
    padding: 10px 10px 10px 10px;
	margin: 0px 0px 0px 0px;
	border:2px solid #4E4E4E;
    font: normal normal normal 14pt/normal 'Open Sans', sans-serif;
	background-color: #B2FFFF;	
}

.protex-button {
    margin-right: 15px;
}

.protex-listbox {
}

.gwt-ProgressBar-shell {
  border: 2px solid #faf9f7;
  border-right: 2px solid #848280;
  border-bottom: 2px solid #848280;
  background-color: #AAAAAA;
  height: 14pt;
  width: 50%;
  margin-left: 15px;
}
.gwt-ProgressBar-shell .gwt-ProgressBar-bar {
  background-color: #67A7E3;
}
.gwt-ProgressBar-shell .gwt-ProgressBar-text {
  padding: 0px;
  margin: 0px;
  color: white;
}

div#protex-panel[style] {
   overflow: scroll !important;
}
