<div id="chemicalequationinput_{{ id }}" class="chemicalequationinput">
    <div class="script_placeholder" data-src="{{ previewer }}" />
    <div class="{{ status.classname }}">
        <input type="text"
               name="input_{{ id }}"
               id="input_{{ id }}"
               aria-label="{{ response_data.label|striptags }}"
               aria-describedby="answer_{{ id }}"
               data-input-id="{{ id }}"
               value="{{ value }}"
               {% if size %}size="{{ size }}"{% endif %} />
        <p class="indicator-container">
            {{ value }}
            {% include "status_span.html" with status=status status_id=id %}
        </p>
        <div id="input_{{ id }}_preview" class="equation"></div>
        <p id="answer_{{ id }}" class="answer"></p>
    </div>
</div>
