{% load static %}
<div class="imageinput capa_inputtype" id="inputtype_{{ id }}">
    <input type="hidden"
           class="imageinput"
           src="{{ src }}"
           name="input_{{ id }}"
           id="input_{{ id }}"
           value="{{ value }}" />
    <div style="position:relative;">
        <div id="imageinput_{{ id }}"
             style="background-image: url('{{ src }}');
                    width: {{ width }}px;
                    height: {{ height }}px;
                    position: relative;
                    left: 0;
                    top: 0">
            <img src="{% static 'images/green-pointer.png' %}"
                 id="cross_{{ id }}"
                 style="position: absolute;
                        top: {{ gy }}px;
                        left: {{ gx }}px"
                 alt="Selection indicator" />
        </div>
        <div data-width="{{ width }}"
             data-height="{{ height }}"
             id="answer_{{ id }}"
             style="position: absolute;
                    left: 0;
                    top: 0"></div>
    </div>
    <script type="text/javascript" charset="utf-8">(new ImageInput('{{ id }}'));</script>
    {% include "status_span.html" with status=status status_id=id %}
</div>
